﻿

var cryptoki = null;
var activeSlot = null;
var token = null;
var session = null;

function showError(msg) {
    $('#error').empty();
    $('#error').append("Error: " + msg);
}

$(window).unload(function () {
    if (session != null) {
        session.logout();
        session.close();
    }

    if (cryptoki != null) {
        cryptoki.Finalize();

        //cryptoki.detach();
    }
});

function initialize() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }

    if ($('#cryptokilib').val() == null || $('#cryptokilib').val() == "") {
        showError("Set the PKCS#11 lib in the related field<br />");
        return false;
    }

    cryptoki = cryptokiFactory.getCryptoki();

    cryptoki.SetLicenseInfo("licensee", "productKey");

    try {
        var res = cryptoki.Attach($('#cryptokilib').val());
        if (res != CryptokiException.CKR_OK) {
            showError("Attach to cryptoki library error: " + res);
            return false;
        }

        res = cryptoki.Initialize();
        if (res != CryptokiException.CKR_OK && res != CryptokiException.CKR_CRYPTOKI_ALREADY_INITIALIZED) {
            showError("Initialize error " + res);
            return false;
        }
    }
    catch (e) {
        showError("Unable to attach to the specified PKCS#11 module. " + e.toString() + '<br />');
        cryptoki = null;
        return false;
    }



    $('#info').append('Initialize OK<br/>')

    return true;
}





function showCryptokiInfo() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized");
        return false;
    }

    try {
        var cryptokiInfo = cryptoki.Info;

        $('#info')
			.append('<br/><b>CryptokiInfo<br/>')
			.append("Manufacturer: " + cryptokiInfo.ManufacturerID + '<br/>')
			.append("Description: " + cryptokiInfo.LibDescription + '<br/>')
			.append("LibVersion: " + cryptokiInfo.LibVersion + '<br/>')
			.append("Version: " + cryptokiInfo.Version + '<br/>');
    }
    catch (e) {
        showError(e.ToString() + '<br />');
        return false;
    }

    return true;
}

function showSlotInfo() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized");
        return false;
    }

    try {
        // Reads the set of slots
        var slots = cryptoki.Slots;
        if (slots.Count == 0) {
            showError("No readers/slots found");
            return false;
        }

        for (i = 0; i < slots.Count; i++) {
            slot = slots.Get(i);

            var slotInfo = slot.Info;

            $('#info')
				.append('<br/><b>SlotInfo<br/>')
        		.append('<b>Slot ' + i + ' :</b>')
        		.append("Description: " + slotInfo.Description + '<br/>')
    			.append("FirmwareVersion: " + slotInfo.FirmwareVersion + '<br/>')
    			.append("HardwareVersion: " + slotInfo.HardwareVersion + '<br/>')
    			.append("Manufacturer: " + slotInfo.ManufacturerID + '<br/>');
        }

        // slots containing a token
        slots = cryptoki.ActiveSlots;

        if (slots.Count == 0) {
            $('#info').
        		append('<br /><b>Insert the smart card in the reader and click again</b><br />');
        }
        else {
            activeSlot = slots.Get(0);
        }
    }
    catch (e) {
        showError(e.ToString() + '<br />');
        return false;
    }

    return true;
}

function showTokenInfo() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized<br />");
        return false;
    }

    if (activeSlot == null) {
        showError("No slot selected. Insert a token and click egain.<br />");
        return false;
    }

    try {
        if (activeSlot.IsTokenPresent) {
            token = activeSlot.Token;

            var tokenInfo = token.Info;

            $('#info')
				.append('<br /><b>TokenInfo</b><br/>')
				.append("Label: " + tokenInfo.Label + '<br/>')
				.append("FirmwareVersion: " + tokenInfo.FirmwareVersion + '<br/>')
				.append("HardwareVersion: " + tokenInfo.HardwareVersion + '<br/>')
				.append("Manufacturer: " + tokenInfo.ManufacturerID + '<br/>');
        }
        else {
            alert('Insert the smart card in the reader and click again.');
        }
    }
    catch (e) {
        showError(e.ToString() + '<br />');
        return false;
    }

    return true;
}

function openSession() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized<br />");
        return false;
    }

    if (token == null) {
        showError("No token available.<br />");
        return false;
    }

    try {
        // Opens a read/write serial session
        session = token.OpenSession(SessionInfo.CKF_SERIAL_SESSION | SessionInfo.CKF_RW_SESSION,
		                                     null,
		                                     null);

        var sessionInfo = session.Info;

        $('#info')
	    	.append('<br /><b>SessionInfo</b><br/>')
			.append("Session Slot ID: " + sessionInfo.SlotID + '<br/>')
			.append("Session State: " + sessionInfo.State + '<br/>');
    }
    catch (e) {
        showError(e.ToString() + '<br />');
        return false;
    }

    return true;
}

function login(pin) {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized<br />");
        return false;
    }

    if (session == null) {
        showError("No session open.<br />");
        return false;
    }

    try {
        if (pin == "") {
            showError("Insert the PIN ");
            return false;
        }
        // Executes the login passing the user PIN
        res = session.Login(Session.CKU_USER, pin);
        if (res != CryptokiException.CKR_OK) {
            showError("Wrong PIN " + res);
            return false;
        }

        $('#info').append('<br />Login OK<br/>')
    }
    catch (e) {
        showError(e.ToString() + '<br />');
        return false;
    }

    return true;
}

function showKeysAndCertificates() {
    if (cryptokiFactory == null) {
        showError("CryptokiApp has not been loaded. Check the Sliverlight or the browser permissions.<br />");
        return false;
    }


    if (cryptoki == null) {
        showError("SLCryptoki not initialized<br />");
        return false;
    }

    if (session == null) {
        showError("No session open.<br />");
        return false;
    }

    try {
        $('#info')
		.append('<br /><b>Objects</b><br/>')

        // Searchs for an RSA private key object
        // Sets the template with its attributes
        var template = new CryptokiTemplate();
        template.add(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PRIVATE_KEY);
        template.add(CryptokiAttribute.CKA_KEY_TYPE, Key.CKK_RSA);

        // Launchs the search specifying the template just created
        var objects = session.Objects.Find(template.getTemplate(), 10);

        for (var i = 0; i < objects.Count; i++) {
            var obj = objects.Get(i);
            $('#info')
    			.append("Private Key " + i + ":" + obj.Label + '<br/>');
        }

        var pubkey = null;
        var prikey = null;

        // gets the private key
        if (objects.Count > 0)
            prikey = objects.Get(0);

        // Searchs for an RSA public key object
        // Sets the template with its attributes
        template = new CryptokiTemplate();
        template.add(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_PUBLIC_KEY);
        template.add(CryptokiAttribute.CKA_KEY_TYPE, Key.CKK_RSA);

        // Launchs the search specifying the template just created
        objects = session.Objects.Find(template.getTemplate(), 10);

        for (var i = 0; i < objects.Count; i++) {
            var obj = objects.Get(i);
            $('#info')
   			.append("Public Key " + i + ":" + obj.Label + '<br/>');

            // gets the public key with the same label as private key
            if (prikey != null && prikey.Label == obj.Label)
                pubkey = obj;
        }

        // Searchs for Certificate object
        // Sets the template with its attributes
        template = new CryptokiTemplate();
        template.add(CryptokiAttribute.CKA_CLASS, CryptokiObject.CKO_CERTIFICATE);

        // Launchs the search specifying the template just created
        objects = session.Objects.Find(template.getTemplate(), 10);

        for (var i = 0; i < objects.Count; i++) {
            var obj = objects.Get(i);
            $('#info')
   			.append("Certificate " + i + ":" + obj.Label + '<br/>');
        }

        if(prikey != null && pubkey != null)
            sign_verify(prikey, pubkey);
        else
            $('#info').append("No keypair found for sign-verify test<br/>");

    }
    catch (e) {
        showError(e.ToString());
        return false;
    }

    return true;
}

function sign_verify(privateKey, publicKey) {

    $('#info')
		.append('<br /><b>Signature</b><br/>')

    $('#info').append("Sign by Private Key: " + privateKey.Label + "<br/>");

    // launches the digital signing operation with a RSA_PKCS mechanism
    var mechanism = new Mechanism(Mechanism.CKM_SHA1_RSA_PKCS);
    res = session.SignInit(mechanism.getMechanism(), privateKey);
    if (res != CryptokiException.CKR_OK) {
        showError("signInit error " + res);
        return false;
    }

    // computes the signature
    var toSign = "ciao ciao".getBytes();
    var toSignByteArray = ConvertToByteArray(toSign);

    var signature = session.Sign(toSignByteArray);

    $('#info').append("Signature OK<br/>");

    $('#info').append("<br/>Verify by public Key: " + publicKey.Label + "<br/>");

    res = session.VerifyInit(mechanism.getMechanism(), publicKey);
    if (res != CryptokiException.CKR_OK) {
        showError("verifyInit error " + res);
        return false;
    }

    res = session.Verify(toSignByteArray, signature);
    if (res != CryptokiException.CKR_OK) {
        showError("Verify NOK " + res);
        return false;
    }

    $('#info').append("Verify OK<br/>");

    return true;
}

function closeAll() {
    session.Logout();

    session.Close();

    cryptoki.Finalize();

    cryptoki.Detach();
}

function dump() {
    $('#error').empty();
    $('#info').empty();

    if (initialize())
        if (showCryptokiInfo())
            if (showSlotInfo())
                if (showTokenInfo())
                    if (openSession()) {
                        var ret = true;
                        var pin = $('#pin').val();
                        if (pin != "")
                            ret = login(pin);

                        if (ret) {
                            showKeysAndCertificates();
                        }

                        closeAll();
                    }
}