/*
 * JCryptoki Javascript
 * Author: Ugo Chirico
 * Blog: http://www.ugochirico.com
 * Company: http://www.cryptware.com
 * Home: http://www.ugosweb.com
 *
 *  Copyright (C) 2013 By Ugo Chirico. All rights reserved
 * 
 *  This script is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License, version 2.1 as published by the Free Software Foundation.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA  02110-1301  USA
 */

/////////////////////////////////
// Session definitions
function Session()
{
};

/* SO user */
Session.CKU_SO = 0;
/* Normal user */
Session.CKU_USER  = 1;
/* Context specific (added in v2.20) */
Session.CKU_CONTEXT_SPECIFIC = 2;

////////////////////////////////////////

/////////////////////////////////////////
// Attribute definitions

function CryptokiAttribute()
{
	
};

CryptokiAttribute.CKA_CLASS                       = 0x00000000;
CryptokiAttribute.CKA_TOKEN                       = 0x00000001;
CryptokiAttribute.CKA_PRIVATE                     = 0x00000002;
CryptokiAttribute.CKA_LABEL                       = 0x00000003;
CryptokiAttribute.CKA_APPLICATION                 = 0x00000010;
CryptokiAttribute.CKA_VALUE                       = 0x00000011;
CryptokiAttribute.CKA_OBJECT_ID                   = 0x00000012;
CryptokiAttribute.CKA_CERTIFICATE_TYPE            = 0x00000080;
CryptokiAttribute.CKA_ISSUER                      = 0x00000081;
CryptokiAttribute.CKA_SERIAL_NUMBER               = 0x00000082;
CryptokiAttribute.CKA_AC_ISSUER                   = 0x00000083;
CryptokiAttribute.CKA_OWNER                       = 0x00000084;
CryptokiAttribute.CKA_ATTR_TYPES                  = 0x00000085;
CryptokiAttribute.CKA_TRUSTED                     = 0x00000086;
CryptokiAttribute.CKA_CERTIFICATE_CATEGORY        = 0x00000087;
CryptokiAttribute.CKA_JAVA_MIDP_SECURITY_DOMAIN   = 0x00000088;
CryptokiAttribute.CKA_URL                         = 0x00000089;
CryptokiAttribute.CKA_HASH_OF_SUBJECT_PUBLIC_KEY  = 0x0000008a;
CryptokiAttribute.CKA_HASH_OF_ISSUER_PUBLIC_KEY   = 0x0000008b;
CryptokiAttribute.CKA_CHECK_VALUE                 = 0x00000090;

CryptokiAttribute.CKA_KEY_TYPE                    = 0x00000100;
CryptokiAttribute.CKA_SUBJECT                     = 0x00000101;
CryptokiAttribute.CKA_ID                          = 0x00000102;
CryptokiAttribute.CKA_SENSITIVE                   = 0x00000103;
CryptokiAttribute.CKA_ENCRYPT                     = 0x00000104;
CryptokiAttribute.CKA_DECRYPT                     = 0x00000105;
CryptokiAttribute.CKA_WRAP                        = 0x00000106;
CryptokiAttribute.CKA_UNWRAP                      = 0x00000107;
CryptokiAttribute.CKA_SIGN                        = 0x00000108;
CryptokiAttribute.CKA_SIGN_RECOVER                = 0x00000109;
CryptokiAttribute.CKA_VERIFY                      = 0x0000010a;
CryptokiAttribute.CKA_VERIFY_RECOVER              = 0x0000010b;
CryptokiAttribute.CKA_DERIVE                      = 0x0000010c;
CryptokiAttribute.CKA_START_DATE                  = 0x00000110;
CryptokiAttribute.CKA_END_DATE                    = 0x00000111;
CryptokiAttribute.CKA_MODULUS                     = 0x00000120;
CryptokiAttribute.CKA_MODULUS_BITS                = 0x00000121;
CryptokiAttribute.CKA_PUBLIC_EXPONENT             = 0x00000122;
CryptokiAttribute.CKA_PRIVATE_EXPONENT            = 0x00000123;
CryptokiAttribute.CKA_PRIME_1                     = 0x00000124;
CryptokiAttribute.CKA_PRIME_2                     = 0x00000125;
CryptokiAttribute.CKA_EXPONENT_1                  = 0x00000126;
CryptokiAttribute.CKA_EXPONENT_2                  = 0x00000127;
CryptokiAttribute.CKA_COEFFICIENT                 = 0x00000128;
CryptokiAttribute.CKA_PRIME                       = 0x00000130;
CryptokiAttribute.CKA_SUBPRIME                    = 0x00000131;
CryptokiAttribute.CKA_BASE                        = 0x00000132;
CryptokiAttribute.CKA_PRIME_BITS                  = 0x00000133;
CryptokiAttribute.CKA_SUBPRIME_BITS               = 0x00000134;
CryptokiAttribute.CKA_VALUE_BITS                  = 0x00000160;
CryptokiAttribute.CKA_VALUE_LEN                   = 0x00000161;
CryptokiAttribute.CKA_EXTRACTABLE                 = 0x00000162;
CryptokiAttribute.CKA_LOCAL                       = 0x00000163;
CryptokiAttribute.CKA_NEVER_EXTRACTABLE           = 0x00000164;
CryptokiAttribute.CKA_ALWAYS_SENSITIVE            = 0x00000165;
CryptokiAttribute.CKA_KEY_GEN_MECHANISM  			= 0x00000166;
CryptokiAttribute.CKA_MODIFIABLE                  = 0x00000170;
CryptokiAttribute.CKA_EC_PARAMS                   = 0x00000180;
CryptokiAttribute.CKA_EC_POINT                    = 0x00000181;
CryptokiAttribute.CKA_SECONDARY_AUTH              = 0x00000200;
CryptokiAttribute.CKA_AUTH_PIN_FLAGS              = 0x00000201;
CryptokiAttribute.CKA_ALWAYS_AUTHENTICATE         = 0x00000202;
CryptokiAttribute.CKA_WRAP_WITH_TRUSTED           = 0x00000210;
//CryptokiAttribute.CKA_WRAP_TEMPLATE               = (CKF_ARRAY_ATTRIBUTE|0x00000211);
//CryptokiAttribute.CKA_UNWRAP_TEMPLATE             = (CKF_ARRAY_ATTRIBUTE|0x00000212);
CryptokiAttribute.CKA_OTP_FORMAT                 =  0x00000220;
CryptokiAttribute.CKA_OTP_LENGTH                  = 0x00000221;
CryptokiAttribute.CKA_OTP_TIME_INTERVAL           = 0x00000222;
CryptokiAttribute.CKA_OTP_USER_FRIENDLY_MODE      = 0x00000223;
CryptokiAttribute.CKA_OTP_CHALLENGE_REQUIREMENT   = 0x00000224;
CryptokiAttribute.CKA_OTP_TIME_REQUIREMENT        = 0x00000225;
CryptokiAttribute.CKA_OTP_COUNTER_REQUIREMENT     = 0x00000226;
CryptokiAttribute.CKA_OTP_PIN_REQUIREMENT         = 0x00000227;
CryptokiAttribute.CKA_OTP_COUNTER                 = 0x0000022e;
CryptokiAttribute.CKA_OTP_TIME                    = 0x0000022f;
CryptokiAttribute.CKA_OTP_USER_IDENTIFIER         = 0x0000022a;
CryptokiAttribute.CKA_OTP_SERVICE_IDENTIFIER      = 0x0000022b;
CryptokiAttribute.CKA_OTP_SERVICE_LOGO            = 0x0000022c;
CryptokiAttribute.CKA_OTP_SERVICE_LOGO_TYPE       = 0x0000022d;


CryptokiAttribute.CKA_HW_FEATURE_TYPE             = 0x00000300;
CryptokiAttribute.CKA_RESET_ON_INIT               = 0x00000301;
CryptokiAttribute.CKA_HAS_RESET                   = 0x00000302;
CryptokiAttribute.CKA_PIXEL_X                     = 0x00000400;
CryptokiAttribute.CKA_PIXEL_Y                     = 0x00000401;
CryptokiAttribute.CKA_RESOLUTION                  = 0x00000402;
CryptokiAttribute.CKA_CHAR_ROWS                   = 0x00000403;
CryptokiAttribute.CKA_CHAR_COLUMNS                = 0x00000404;
CryptokiAttribute.CKA_COLOR                       = 0x00000405;
CryptokiAttribute.CKA_BITS_PER_PIXEL              = 0x00000406;
CryptokiAttribute.CKA_CHAR_SETS                   = 0x00000480;
CryptokiAttribute.CKA_ENCODING_METHODS            = 0x00000481;
CryptokiAttribute.CKA_MIME_TYPES                  = 0x00000482;
CryptokiAttribute.CKA_MECHANISM_TYPE              = 0x00000500;
CryptokiAttribute.CKA_REQUIRED_CMS_ATTRIBUTES     = 0x00000501;
CryptokiAttribute.CKA_DEFAULT_CMS_ATTRIBUTES      = 0x00000502;
CryptokiAttribute.CKA_SUPPORTED_CMS_ATTRIBUTES    = 0x00000503;
//CryptokiAttribute.CKA_ALLOWED_MECHANISMS          = (CKF_ARRAY_ATTRIBUTE|0x00000600);

// ERACOM defined values
// Eracom PTK
CryptokiAttribute.ERACOM_PTK_USAGE_COUNT      = 0x80000101;
CryptokiAttribute.ERACOM_PTK_TIME_STAMP       = 0x80000102;
CryptokiAttribute.ERACOM_PTK_CHECK_VALUE      = 0x80000103;
CryptokiAttribute.ERACOM_PTK_MECHANISM_LIST   = 0x80000104;
CryptokiAttribute.ERACOM_PTK_SIGN_LOCAL_CERT  = 0x80000127;
CryptokiAttribute.ERACOM_PTK_EXPORT           = 0x80000128;
CryptokiAttribute.ERACOM_PTK_EXPORTABLE       = 0x80000129;
CryptokiAttribute.ERACOM_PTK_DELETABLE        = 0x8000012a;
CryptokiAttribute.ERACOM_PTK_IMPORT           = 0x8000012b;
CryptokiAttribute.ERACOM_PTK_KEY_SIZE         = 0x8000012c;
CryptokiAttribute.ERACOM_PTK_ISSUER_STR       = 0x80000130;
CryptokiAttribute.ERACOM_PTK_SUBJECT_STR      = 0x80000131;
CryptokiAttribute.ERACOM_PTK_SERIAL_NUMBER_INT = 0x80000132;
CryptokiAttribute.ERACOM_PTK_RECORD_COUNT     = 0x80000136;
CryptokiAttribute.ERACOM_PTK_RECORD_NUMBER    = 0x80000137;
CryptokiAttribute.ERACOM_PTK_PURGE            = 0x80000139;
CryptokiAttribute.ERACOM_PTK_EVENT_LOG_FULL   = 0x8000013a;
CryptokiAttribute.ERACOM_PTK_SECURITY_MODE    = 0x80000140;
CryptokiAttribute.ERACOM_PTK_TRANSPORT_MODE   = 0x80000141;
CryptokiAttribute.ERACOM_PTK_BATCH            = 0x80000142;
CryptokiAttribute.ERACOM_PTK_HW_STATUS        = 0x80000143;
CryptokiAttribute.ERACOM_PTK_FREE_MEM         = 0x80000144;
CryptokiAttribute.ERACOM_PTK_TAMPER_CMD       = 0x80000145;
CryptokiAttribute.ERACOM_PTK_DATE_OF_MANUFACTURE = 0x80000146;
CryptokiAttribute.ERACOM_PTK_HALT_CMD         = 0x80000147;
CryptokiAttribute.ERACOM_PTK_APPLICATION_COUNT = 0x80000148;
CryptokiAttribute.ERACOM_PTK_FW_VERSION       = 0x80000149;
CryptokiAttribute.ERACOM_PTK_RESCAN_PERIPHERALS_CMD = 0x8000014a;
CryptokiAttribute.ERACOM_PTK_RTC_AAC_ENABLED  = 0x8000014b;
CryptokiAttribute.ERACOM_PTK_RTC_AAC_GUARD_SECONDS = 0x8000014c;
CryptokiAttribute.ERACOM_PTK_RTC_AAC_GUARD_COUNT = 0x8000014d;
CryptokiAttribute.ERACOM_PTK_RTC_AAC_GUARD_DURATION = 0x8000014e;
CryptokiAttribute.ERACOM_PTK_HW_EXT_INFO_STR  = 0x8000014f;
CryptokiAttribute.ERACOM_PTK_SLOT_ID          = 0x80000151;
CryptokiAttribute.ERACOM_PTK_MAX_SESSIONS     = 0x80000155;
CryptokiAttribute.ERACOM_PTK_MIN_PIN_LEN      = 0x80000156;
CryptokiAttribute.ERACOM_PTK_MAX_PIN_FAIL     = 0x80000158;
CryptokiAttribute.ERACOM_PTK_FLAGS            = 0x80000159;
CryptokiAttribute.ERACOM_PTK_VERIFY_OS        = 0x80000170;
CryptokiAttribute.ERACOM_PTK_VERSION          = 0x80000181;
CryptokiAttribute.ERACOM_PTK_MANUFACTURER     = 0x80000182;
CryptokiAttribute.ERACOM_PTK_BUILD_DATE       = 0x80000183;
CryptokiAttribute.ERACOM_PTK_FINGERPRINT      = 0x80000184;
CryptokiAttribute.ERACOM_PTK_ROM_SPACE        = 0x80000185;
CryptokiAttribute.ERACOM_PTK_RAM_SPACE        = 0x80000186;
CryptokiAttribute.ERACOM_PTK_FM_STATUS        = 0x80000187;
CryptokiAttribute.ERACOM_PTK_DELETE_FM        = 0x80000188;
CryptokiAttribute.ERACOM_PTK_FM_STARTUP_STATUS = 0x80000189;
CryptokiAttribute.ERACOM_PTK_CERTIFICATE_START_TIME = 0x80000190;
CryptokiAttribute.ERACOM_PTK_CERTIFICATE_END_TIME = 0x80000191;
CryptokiAttribute.ERACOM_PTK_PKI_ATTRIBUTE_BER_ENCODED = 0x80000230;
CryptokiAttribute.ERACOM_PTK_HIFACE_MASTER    = 0x80000250;
CryptokiAttribute.ERACOM_PTK_CKA_SEED         = 0x80000260;
CryptokiAttribute.ERACOM_PTK_CKA_COUNTER      = 0x80000261;
CryptokiAttribute.ERACOM_PTK_CKA_H_VALUE      = 0x80000262;
CryptokiAttribute.ERACOM_PTK_ENUM_ATTRIBUTE   = 0x0000ffff;

//////////////////////////////////////////////////////

function CryptokiTemplate()
{
	this.template = cryptokiHelper.CreateTemplate();
	this.add = function (type, value) 
		{
	        cryptokiHelper.AddToTemplate(this.template, type, value);
		};
	
	this.getTemplate = function () { return this.template;};
};

//////////////////////////////////////////////////////
// TokenInfo definitions

function TokenInfo() 
{
};

TokenInfo.CKF_RNG                   = 0x00000001;
TokenInfo.CKF_WRITE_PROTECTED       = 0x00000002;
TokenInfo.CKF_LOGIN_REQUIRED        = 0x00000004;
TokenInfo.CKF_USER_PIN_INITIALIZED  = 0x00000008;
TokenInfo.CKF_RESTORE_KEY_NOT_NEEDED=  0x00000020;
TokenInfo.CKF_CLOCK_ON_TOKEN        =  0x00000040;
TokenInfo.CKF_PROTECTED_AUTHENTICATION_PATH =0x00000100;
TokenInfo.CKF_DUAL_CRYPTO_OPERATIONS  =0x00000200;
TokenInfo.CKF_TOKEN_INITIALIZED       =0x00000400;
TokenInfo.CKF_SECONDARY_AUTHENTICATION = 0x00000800;
TokenInfo.CKF_USER_PIN_COUNT_LOW       =0x00010000;
TokenInfo.CKF_USER_PIN_FINAL_TRY       =0x00020000;
TokenInfo.CKF_USER_PIN_LOCKED          =0x00040000;
TokenInfo.CKF_USER_PIN_TO_BE_CHANGED   =0x00080000;
TokenInfo.CKF_SO_PIN_COUNT_LOW         =0x00100000;
TokenInfo.CKF_SO_PIN_FINAL_TRY         =0x00200000;
TokenInfo.CKF_SO_PIN_LOCKED            =0x00400000;
TokenInfo.CKF_SO_PIN_TO_BE_CHANGED     =0x00800000;
///////////////////////////////////////////////////

///////////////////////////////////////////////////
// SessionInfo definitions

function SessionInfo()
{
};

SessionInfo.CKF_RW_SESSION = 0x00000002;  /* session is r/w */
SessionInfo.CKF_SERIAL_SESSION = 0x00000004;  /* no parallel */

SessionInfo.CKF_RO_PUBLIC_SESSION = 0;
SessionInfo.CKF_RO_USER_FUNCTIONS = 1;
SessionInfo.CKF_RW_PUBLIC_SESSION = 2;
SessionInfo.CKF_RW_USER_FUNCTIONS = 3;
SessionInfo.CKF_RW_SO_FUNCTIONS   = 4;
///////////////////////////////////////////////////
///////////////////////////////////////////////////
// SlotInfo definitions

function SlotInfo()
{
};

SlotInfo.CKF_TOKEN_PRESENT    = 0x00000001;
SlotInfo.CKF_REMOVABLE_DEVICE = 0x00000002;
SlotInfo.CKF_HW_SLOT          = 0x00000004;

///////////////////////////////////////////////////
// Mechanism definitions
function Mechanism(type, value)
{
	this.mechanism = cryptokiHelper.CreateMechanism(type, value);
};

function Mechanism(type)
{
    this.mechanism = cryptokiHelper.CreateMechanism(type);
};

Mechanism.prototype.getMechanism = function() { return this.mechanism;};


Mechanism.CKM_RSA_PKCS_KEY_PAIR_GEN       = 0x00000000;
Mechanism.CKM_RSA_PKCS                    = 0x00000001;
Mechanism.CKM_RSA_9796                    = 0x00000002;
Mechanism.CKM_RSA_X_509                   = 0x00000003;
Mechanism.CKM_MD2_RSA_PKCS                = 0x00000004;
Mechanism.CKM_MD5_RSA_PKCS                = 0x00000005;
Mechanism.CKM_SHA1_RSA_PKCS               = 0x00000006;
Mechanism.CKM_RIPEMD128_RSA_PKCS          = 0x00000007;
Mechanism.CKM_RIPEMD160_RSA_PKCS          = 0x00000008;
Mechanism.CKM_RSA_PKCS_OAEP               = 0x00000009;
Mechanism.CKM_RSA_X9_31_KEY_PAIR_GEN      = 0x0000000a;
Mechanism.CKM_RSA_X9_31                   = 0x0000000b;
Mechanism.CKM_SHA1_RSA_X9_31              = 0x0000000c;
Mechanism.CKM_RSA_PKCS_PSS                = 0x0000000d;
Mechanism.CKM_SHA1_RSA_PKCS_PSS           = 0x0000000e;
Mechanism.CKM_DSA_KEY_PAIR_GEN            = 0x00000010;
Mechanism.CKM_DSA                         = 0x00000011;
Mechanism.CKM_DSA_SHA1                    = 0x00000012;
Mechanism.CKM_DH_PKCS_KEY_PAIR_GEN        = 0x00000020;
Mechanism.CKM_DH_PKCS_DERIVE              = 0x00000021;
Mechanism.CKM_X9_42_DH_KEY_PAIR_GEN       = 0x00000030;
Mechanism.CKM_X9_42_DH_DERIVE             = 0x00000031;
Mechanism.CKM_X9_42_DH_HYBRID_DERIVE      = 0x00000032;
Mechanism.CKM_X9_42_MQV_DERIVE            = 0x00000033;
Mechanism.CKM_SHA256_RSA_PKCS             = 0x00000040;
Mechanism.CKM_SHA384_RSA_PKCS             = 0x00000041;
Mechanism.CKM_SHA512_RSA_PKCS             = 0x00000042;
Mechanism.CKM_SHA256_RSA_PKCS_PSS         = 0x00000043;
Mechanism.CKM_SHA384_RSA_PKCS_PSS         = 0x00000044;
Mechanism.CKM_SHA512_RSA_PKCS_PSS         = 0x00000045;
Mechanism.CKM_SHA224_RSA_PKCS             = 0x00000046;
Mechanism.CKM_SHA224_RSA_PKCS_PSS         = 0x00000047;
Mechanism.CKM_RC2_KEY_GEN                 = 0x00000100;
Mechanism.CKM_RC2_ECB                     = 0x00000101;
Mechanism.CKM_RC2_CBC                     = 0x00000102;
Mechanism.CKM_RC2_MAC                     = 0x00000103;
Mechanism.CKM_RC2_MAC_GENERAL             = 0x00000104;
Mechanism.CKM_RC2_CBC_PAD                 = 0x00000105;
Mechanism.CKM_RC4_KEY_GEN                 = 0x00000110;
Mechanism.CKM_RC4                         = 0x00000111;
Mechanism.CKM_DES_KEY_GEN                 = 0x00000120;
Mechanism.CKM_DES_ECB                     = 0x00000121;
Mechanism.CKM_DES_CBC                     = 0x00000122;
Mechanism.CKM_DES_MAC                     = 0x00000123;
Mechanism.CKM_DES_MAC_GENERAL             = 0x00000124;
Mechanism.CKM_DES_CBC_PAD                 = 0x00000125;
Mechanism.CKM_DES2_KEY_GEN                = 0x00000130;
Mechanism.CKM_DES3_KEY_GEN                = 0x00000131;
Mechanism.CKM_DES3_ECB                    = 0x00000132;
Mechanism.CKM_DES3_CBC                    = 0x00000133;
Mechanism.CKM_DES3_MAC                    = 0x00000134;
Mechanism.CKM_DES3_MAC_GENERAL            = 0x00000135;
Mechanism.CKM_DES3_CBC_PAD                = 0x00000136;
Mechanism.CKM_CDMF_KEY_GEN                = 0x00000140;
Mechanism.CKM_CDMF_ECB                    = 0x00000141;
Mechanism.CKM_CDMF_CBC                    = 0x00000142;
Mechanism.CKM_CDMF_MAC                    = 0x00000143;
Mechanism.CKM_CDMF_MAC_GENERAL            = 0x00000144;
Mechanism.CKM_CDMF_CBC_PAD                = 0x00000145;
Mechanism.CKM_DES_OFB64                   = 0x00000150;
Mechanism.CKM_DES_OFB8                    = 0x00000151;
Mechanism.CKM_DES_CFB64                   = 0x00000152;
Mechanism.CKM_DES_CFB8                    = 0x00000153;
Mechanism.CKM_MD2                         = 0x00000200;
Mechanism.CKM_MD2_HMAC                    = 0x00000201;
Mechanism.CKM_MD2_HMAC_GENERAL            = 0x00000202;
Mechanism.CKM_MD5                         = 0x00000210;
Mechanism.CKM_MD5_HMAC                    = 0x00000211;
Mechanism.CKM_MD5_HMAC_GENERAL            = 0x00000212;
Mechanism.CKM_SHA_1                       = 0x00000220;
Mechanism.CKM_SHA_1_HMAC                  = 0x00000221;
Mechanism.CKM_SHA_1_HMAC_GENERAL          = 0x00000222;
Mechanism.CKM_RIPEMD128                   = 0x00000230;
Mechanism.CKM_RIPEMD128_HMAC              = 0x00000231;
Mechanism.CKM_RIPEMD128_HMAC_GENERAL      = 0x00000232;
Mechanism.CKM_RIPEMD160                   = 0x00000240;
Mechanism.CKM_RIPEMD160_HMAC              = 0x00000241;
Mechanism.CKM_RIPEMD160_HMAC_GENERAL      = 0x00000242;
Mechanism.CKM_SHA256                      = 0x00000250;
Mechanism.CKM_SHA256_HMAC                 = 0x00000251;
Mechanism.CKM_SHA256_HMAC_GENERAL         = 0x00000252;
Mechanism.CKM_SHA384                      = 0x00000260;
Mechanism.CKM_SHA384_HMAC                 = 0x00000261;
Mechanism.CKM_SHA384_HMAC_GENERAL         = 0x00000262;
Mechanism.CKM_SHA512                      = 0x00000270;
Mechanism.CKM_SHA512_HMAC                 = 0x00000271;
Mechanism.CKM_SHA512_HMAC_GENERAL         = 0x00000272;
Mechanism.CKM_SECURID_KEY_GEN             = 0x00000280;
Mechanism.CKM_SECURID                     = 0x00000282;
Mechanism.CKM_HOTP_KEY_GEN                = 0x00000290;
Mechanism.CKM_HOTP                        = 0x00000291;
Mechanism.CKM_ACTI                        = 0x000002a0;
Mechanism.CKM_ACTI_KEY_GEN                = 0x000002a1;
Mechanism.CKM_CAST_KEY_GEN                = 0x00000300;
Mechanism.CKM_CAST_ECB                    = 0x00000301;
Mechanism.CKM_CAST_CBC                    = 0x00000302;
Mechanism.CKM_CAST_MAC                    = 0x00000303;
Mechanism.CKM_CAST_MAC_GENERAL            = 0x00000304;
Mechanism.CKM_CAST_CBC_PAD                = 0x00000305;
Mechanism.CKM_CAST3_KEY_GEN               = 0x00000310;
Mechanism.CKM_CAST3_ECB                   = 0x00000311;
Mechanism.CKM_CAST3_CBC                   = 0x00000312;
Mechanism.CKM_CAST3_MAC                   = 0x00000313;
Mechanism.CKM_CAST3_MAC_GENERAL           = 0x00000314;
Mechanism.CKM_CAST3_CBC_PAD               = 0x00000315;
Mechanism.CKM_CAST5_KEY_GEN               = 0x00000320;
Mechanism.CKM_CAST5_ECB                   = 0x00000321;
Mechanism.CKM_CAST5_CBC                   = 0x00000322;
Mechanism.CKM_CAST5_MAC                   = 0x00000323;
Mechanism.CKM_CAST5_MAC_GENERAL           = 0x00000324;
Mechanism.CKM_CAST5_CBC_PAD               = 0x00000325;
Mechanism.CKM_CAST128_KEY_GEN             = 0x00000320;
Mechanism.CKM_CAST128_ECB                 = 0x00000321;
Mechanism.CKM_CAST128_CBC                 = 0x00000322;
Mechanism.CKM_CAST128_MAC                 = 0x00000323;
Mechanism.CKM_CAST128_MAC_GENERAL         = 0x00000324;
Mechanism.CKM_CAST128_CBC_PAD             = 0x00000325;
Mechanism.CKM_RC5_KEY_GEN                 = 0x00000330;
Mechanism.CKM_RC5_ECB                     = 0x00000331;
Mechanism.CKM_RC5_CBC                     = 0x00000332;
Mechanism.CKM_RC5_MAC                     = 0x00000333;
Mechanism.CKM_RC5_MAC_GENERAL             = 0x00000334;
Mechanism.CKM_RC5_CBC_PAD                 = 0x00000335;
Mechanism.CKM_IDEA_KEY_GEN                = 0x00000340;
Mechanism.CKM_IDEA_ECB                    = 0x00000341;
Mechanism.CKM_IDEA_CBC                    = 0x00000342;
Mechanism.CKM_IDEA_MAC                    = 0x00000343;
Mechanism.CKM_IDEA_MAC_GENERAL            = 0x00000344;
Mechanism.CKM_IDEA_CBC_PAD                = 0x00000345;
Mechanism.CKM_GENERIC_SECRET_KEY_GEN      = 0x00000350;
Mechanism.CKM_CONCATENATE_BASE_AND_KEY    = 0x00000360;
Mechanism.CKM_CONCATENATE_BASE_AND_DATA   = 0x00000362;
Mechanism.CKM_CONCATENATE_DATA_AND_BASE   = 0x00000363;
Mechanism.CKM_XOR_BASE_AND_DATA           = 0x00000364;
Mechanism.CKM_EXTRACT_KEY_FROM_KEY        = 0x00000365;
Mechanism.CKM_SSL3_PRE_MASTER_KEY_GEN     = 0x00000370;
Mechanism.CKM_SSL3_MASTER_KEY_DERIVE      = 0x00000371;
Mechanism.CKM_SSL3_KEY_AND_MAC_DERIVE     = 0x00000372;
Mechanism.CKM_SSL3_MASTER_KEY_DERIVE_DH   = 0x00000373;
Mechanism.CKM_TLS_PRE_MASTER_KEY_GEN      = 0x00000374;
Mechanism.CKM_TLS_MASTER_KEY_DERIVE       = 0x00000375;
Mechanism.CKM_TLS_KEY_AND_MAC_DERIVE      = 0x00000376;
Mechanism.CKM_TLS_MASTER_KEY_DERIVE_DH    = 0x00000377;
Mechanism.CKM_SSL3_MD5_MAC                = 0x00000380;
Mechanism.CKM_SSL3_SHA1_MAC               = 0x00000381;
Mechanism.CKM_MD5_KEY_DERIVATION          = 0x00000390;
Mechanism.CKM_MD2_KEY_DERIVATION          = 0x00000391;
Mechanism.CKM_SHA1_KEY_DERIVATION         = 0x00000392;
Mechanism.CKM_SHA256_KEY_DERIVATION       = 0x00000393;
Mechanism.CKM_SHA384_KEY_DERIVATION       = 0x00000394;
Mechanism.CKM_SHA512_KEY_DERIVATION       = 0x00000395;
Mechanism.CKM_PBE_MD2_DES_CBC             = 0x000003a0;
Mechanism.CKM_PBE_MD5_DES_CBC             = 0x000003a1;
Mechanism.CKM_PBE_MD5_CAST_CBC            = 0x000003a2;
Mechanism.CKM_PBE_MD5_CAST3_CBC           = 0x000003a3;
Mechanism.CKM_PBE_MD5_CAST5_CBC           = 0x000003a4;
Mechanism.CKM_PBE_MD5_CAST128_CBC         = 0x000003a4;
Mechanism.CKM_PBE_SHA1_CAST5_CBC          = 0x000003a5;
Mechanism.CKM_PBE_SHA1_CAST128_CBC        = 0x000003a5;
Mechanism.CKM_PBE_SHA1_RC4_128            = 0x000003a6;
Mechanism.CKM_PBE_SHA1_RC4_40             = 0x000003a7;
Mechanism.CKM_PBE_SHA1_DES3_EDE_CBC       = 0x000003a8;
Mechanism.CKM_PBE_SHA1_DES2_EDE_CBC       = 0x000003a9;
Mechanism.CKM_PBE_SHA1_RC2_128_CBC        = 0x000003aa;
Mechanism.CKM_PBE_SHA1_RC2_40_CBC         = 0x000003ab;
Mechanism.CKM_PKCS5_PBKD2                 = 0x000003b0;
Mechanism.CKM_PBA_SHA1_WITH_SHA1_HMAC     = 0x000003c0;
Mechanism.CKM_WTLS_PRE_MASTER_KEY_GEN     = 0x000003d0;
Mechanism.CKM_WTLS_MASTER_KEY_DERIVE      = 0x000003d1;
Mechanism.CKM_WTLS_MASTER_KEY_DERIVE_DH_ECC = 0x000003d2;
Mechanism.CKM_WTLS_PRF                    = 0x000003d3;
Mechanism.CKM_WTLS_SERVER_KEY_AND_MAC_DERIVE = 0x000003d4;
Mechanism.CKM_WTLS_CLIENT_KEY_AND_MAC_DERIVE = 0x000003d5;
Mechanism.CKM_KEY_WRAP_LYNKS              = 0x00000400;
Mechanism.CKM_KEY_WRAP_SET_OAEP           = 0x00000401;
Mechanism.CKM_CMS_SIG                     = 0x00000500;
Mechanism.CKM_KIP_DERIVE                  = 0x00000510;
Mechanism.CKM_KIP_WRAP                    = 0x00000511;
Mechanism.CKM_KIP_MAC                     = 0x00000512;
Mechanism.CKM_CAMELLIA_KEY_GEN            = 0x00000550;
Mechanism.CKM_CAMELLIA_ECB                = 0x00000551;
Mechanism.CKM_CAMELLIA_CBC                = 0x00000552;
Mechanism.CKM_CAMELLIA_MAC                = 0x00000553;
Mechanism.CKM_CAMELLIA_MAC_GENERAL        = 0x00000554;
Mechanism.CKM_CAMELLIA_CBC_PAD            = 0x00000555;
Mechanism.CKM_CAMELLIA_ECB_ENCRYPT_DATA   = 0x00000556;
Mechanism.CKM_CAMELLIA_CBC_ENCRYPT_DATA   = 0x00000557;
Mechanism.CKM_CAMELLIA_CTR                = 0x00000558;
Mechanism.CKM_ARIA_KEY_GEN                = 0x00000560;
Mechanism.CKM_ARIA_ECB                    = 0x00000561;
Mechanism.CKM_ARIA_CBC                    = 0x00000562;
Mechanism.CKM_ARIA_MAC                    = 0x00000563;
Mechanism.CKM_ARIA_MAC_GENERAL            = 0x00000564;
Mechanism.CKM_ARIA_CBC_PAD                = 0x00000565;
Mechanism.CKM_ARIA_ECB_ENCRYPT_DATA       = 0x00000566;
Mechanism.CKM_ARIA_CBC_ENCRYPT_DATA       = 0x00000567;

Mechanism.CKM_SKIPJACK_KEY_GEN            = 0x00001000;
Mechanism.CKM_SKIPJACK_ECB64              = 0x00001001;
Mechanism.CKM_SKIPJACK_CBC64              = 0x00001002;
Mechanism.CKM_SKIPJACK_OFB64              = 0x00001003;
Mechanism.CKM_SKIPJACK_CFB64              = 0x00001004;
Mechanism.CKM_SKIPJACK_CFB32              = 0x00001005;
Mechanism.CKM_SKIPJACK_CFB16              = 0x00001006;
Mechanism.CKM_SKIPJACK_CFB8               = 0x00001007;
Mechanism.CKM_SKIPJACK_WRAP               = 0x00001008;
Mechanism.CKM_SKIPJACK_PRIVATE_WRAP       = 0x00001009;
Mechanism.CKM_SKIPJACK_RELAYX             = 0x0000100a;
Mechanism.CKM_KEA_KEY_PAIR_GEN            = 0x00001010;
Mechanism.CKM_KEA_KEY_DERIVE              = 0x00001011;
Mechanism.CKM_FORTEZZA_TIMESTAMP          = 0x00001020;
Mechanism.CKM_BATON_KEY_GEN               = 0x00001030;
Mechanism.CKM_BATON_ECB128                = 0x00001031;
Mechanism.CKM_BATON_ECB96                 = 0x00001032;
Mechanism.CKM_BATON_CBC128                = 0x00001033;
Mechanism.CKM_BATON_COUNTER               = 0x00001034;
Mechanism.CKM_BATON_SHUFFLE               = 0x00001035;
Mechanism.CKM_BATON_WRAP                  = 0x00001036;
Mechanism.CKM_ECDSA_KEY_PAIR_GEN          = 0x00001040;
Mechanism.CKM_EC_KEY_PAIR_GEN             = 0x00001040;
Mechanism.CKM_ECDSA                       = 0x00001041;
Mechanism.CKM_ECDSA_SHA1                  = 0x00001042;
Mechanism.CKM_ECDH1_DERIVE                = 0x00001050;
Mechanism.CKM_ECDH1_COFACTOR_DERIVE       = 0x00001051;
Mechanism.CKM_ECMQV_DERIVE                = 0x00001052;

Mechanism.CKM_JUNIPER_KEY_GEN             = 0x00001060;
Mechanism.CKM_JUNIPER_ECB128              = 0x00001061;
Mechanism.CKM_JUNIPER_CBC128              = 0x00001062;
Mechanism.CKM_JUNIPER_COUNTER             = 0x00001063;
Mechanism.CKM_JUNIPER_SHUFFLE             = 0x00001064;
Mechanism.CKM_JUNIPER_WRAP                = 0x00001065;
Mechanism.CKM_FASTHASH                    = 0x00001070;
Mechanism.CKM_AES_KEY_GEN                 = 0x00001080;
Mechanism.CKM_AES_ECB                     = 0x00001081;
Mechanism.CKM_AES_CBC                     = 0x00001082;
Mechanism.CKM_AES_MAC                     = 0x00001083;
Mechanism.CKM_AES_MAC_GENERAL             = 0x00001084;
Mechanism.CKM_AES_CBC_PAD                 = 0x00001085;
Mechanism.CKM_DES_ECB_ENCRYPT_DATA        = 0x00001100;
Mechanism.CKM_DES_CBC_ENCRYPT_DATA        = 0x00001101;
Mechanism.CKM_DES3_ECB_ENCRYPT_DATA       = 0x00001102;
Mechanism.CKM_DES3_CBC_ENCRYPT_DATA       = 0x00001103;
Mechanism.CKM_AES_ECB_ENCRYPT_DATA        = 0x00001104;
Mechanism.CKM_AES_CBC_ENCRYPT_DATA        = 0x00001105;

Mechanism.CKM_DSA_PARAMETER_GEN           = 0x00002000;
Mechanism.CKM_DH_PKCS_PARAMETER_GEN       = 0x00002001;
Mechanism.CKM_X9_42_DH_PARAMETER_GEN      = 0x00002002;

// ERACOM defined values
// Eracom PTK
Mechanism.ERACOM_PTK_DSA_SHA1_PKCS    = 0x80000000 + Mechanism.CKM_DSA_SHA1 + 1;
Mechanism.ERACOM_PTK_KEY_TRANSLATION  = 0x8000001b;
Mechanism.ERACOM_PTK_RC2_ECB_PAD      = 0x80000000 + Mechanism.CKM_RC2_ECB;
Mechanism.ERACOM_PTK_DES_ECB_PAD      = 0x80000000 + Mechanism.CKM_DES_ECB;
Mechanism.ERACOM_PTK_DES3_ECB_PAD     = 0x80000000 + Mechanism.CKM_DES3_ECB;
Mechanism.ERACOM_PTK_DES3_X919_MAC    = 0x80000000 + Mechanism.CKM_DES3_MAC;
Mechanism.ERACOM_PTK_DES3_X919_MAC_GENERAL = 0x80000000 + Mechanism.CKM_DES3_MAC_GENERAL;
Mechanism.ERACOM_PTK_DES_MDC_2_PAD1   = 0x80000200;
Mechanism.ERACOM_PTK_ARDFP            = 0x80000204;
Mechanism.ERACOM_PTK_NVB              = 0x80000205;
Mechanism.ERACOM_PTK_CAST5_ECB_PAD    = 0x80000000 + Mechanism.CKM_CAST5_ECB;
Mechanism.ERACOM_PTK_CAST128_ECB_PAD  = Mechanism.ERACOM_PTK_CAST5_ECB_PAD;
Mechanism.ERACOM_PTK_IDEA_ECB_PAD     = 0x80000000 + Mechanism.CKM_IDEA_ECB;
Mechanism.ERACOM_PTK_XOR_BASE_AND_KEY = 0x80000364;
Mechanism.ERACOM_PTK_DES_BCFv         = 0x8000038e;
Mechanism.ERACOM_PTK_DES3_BCF         = 0x8000038f;
Mechanism.ERACOM_PTK_DES_DERIVE_ECB   = 0x80000500;
Mechanism.ERACOM_PTK_DES_DERIVE_CBC   = 0x80000501;
Mechanism.ERACOM_PTK_DES3_DERIVE_ECB  = 0x80000502;
Mechanism.ERACOM_PTK_DES3_DERIVE_CBC  = 0x80000503;
Mechanism.ERACOM_PTK_DES3_RETAIL_CFB_MAC = 0x80000510;
Mechanism.ERACOM_PTK_SHA1_RSA_PKCS_TIMESTAMP = 0x80000600;
Mechanism.ERACOM_PTK_DECODE_PKCS_7    = 0x80000935;
Mechanism.ERACOM_PTK_DES_OFB64        = 0x80000940;
Mechanism.ERACOM_PTK_DES3_OFB64       = 0x80000941;
Mechanism.ERACOM_PTK_ENCODE_ATTRIBUTES = 0x80000950;
Mechanism.ERACOM_PTK_ENCODE_X_509     = 0x80000951;
Mechanism.ERACOM_PTK_ENCODE_PKCS_10   = 0x80000952;
Mechanism.ERACOM_PTK_DECODE_X_509     = 0x80000953;
Mechanism.ERACOM_PTK_ENCODE_PUBLIC_KEY = 0x80000954;
Mechanism.ERACOM_PTK_ENCODE_X_509_LOCAL_CERT = 0x80000955;
Mechanism.ERACOM_PTK_WRAPKEY_DES3_ECB = 0x80000961;
Mechanism.ERACOM_PTK_WRAPKEY_DES3_CBC = 0x80000962;
Mechanism.ERACOM_PTK_DES3_DDD_CBC     = 0x80000964;
Mechanism.ERACOM_PTK_OS_UPGRADE       = 0x80000990;
Mechanism.ERACOM_PTK_FM_DOWNLOAD      = 0x80000991;
Mechanism.ERACOM_PTK_PP_LOAD_SECRET   = 0x800009a0;
Mechanism.ERACOM_PTK_VISA_CVV         = 0x800009b0;
Mechanism.ERACOM_PTK_ZKA_MDC_2_KEY_DERIVATION = 0x800009c0;
Mechanism.ERACOM_PTK_SEED_KEY_GEN     = 0x800009d0;
Mechanism.ERACOM_PTK_SEED_ECB         = 0x800009d1;
Mechanism.ERACOM_PTK_SEED_CBC         = 0x800009d2;
Mechanism.ERACOM_PTK_SEED_MAC         = 0x800009d3;
Mechanism.ERACOM_PTK_SEED_MAC_GENERAL = 0x800009d4;
Mechanism.ERACOM_PTK_SEED_ECB_PAD     = 0x800009d5;
Mechanism.ERACOM_PTK_SEED_CBC_PAD     = 0x800009d6;
Mechanism.ERACOM_PTK_REPLICATE_TOKEN_RSA_AES = 0x800009e0;
Mechanism.ERACOM_PTK_SECRET_SHARE_WITH_ATTRIBUTES = 0x800009f0;
Mechanism.ERACOM_PTK_SECRET_RECOVER_WITH_ATTRIBUTES = 0x800009f1;
Mechanism.ERACOM_PTK_PKCS12_PBE_EXPORT = 0x800009f2;
Mechanism.ERACOM_PTK_PKCS12_PBE_IMPORT = 0x800009f3;
Mechanism.ERACOM_PTK_ECIES            = 0x80000a00;

/////////////////////////////////////////////////////////
// CryptokiObject definitions
function CryptokiObject()
{
};

CryptokiObject.CKO_DATA                = 0x00000000;
CryptokiObject.CKO_CERTIFICATE         = 0x00000001;
CryptokiObject.CKO_PUBLIC_KEY          = 0x00000002;
CryptokiObject.CKO_PRIVATE_KEY         = 0x00000003;
CryptokiObject.CKO_SECRET_KEY          = 0x00000004;
CryptokiObject.CKO_HW_FEATURE          = 0x00000005;
CryptokiObject.CKO_DOMAIN_PARAMETERS   = 0x00000006;
CryptokiObject.CKO_MECHANISM           = 0x00000007;
CryptokiObject.CKO_OTP_KEY             = 0x00000008;
//////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
// Key Definitions

function Key()
{
};

Key.CKK_RSA             = 0x00000000;
Key.CKK_DSA             = 0x00000001;
Key.CKK_DH              = 0x00000002;
Key.CKK_EC              = 0x00000003;
Key.CKK_X9_42_DH        = 0x00000004;
Key.CKK_KEA             = 0x00000005;
Key.CKK_GENERIC_SECRET  = 0x00000010;
Key.CKK_RC2             = 0x00000011;
Key.CKK_RC4             = 0x00000012;
Key.CKK_DES             = 0x00000013;
Key.CKK_DES2            = 0x00000014;
Key.CKK_DES3            = 0x00000015;
Key.CKK_CAST            = 0x00000016;
Key.CKK_CAST3           = 0x00000017;
Key.CKK_CAST128         = 0x00000018;
Key.CKK_RC5             = 0x00000019;
Key.CKK_IDEA            = 0x0000001a;
Key.CKK_SKIPJACK        = 0x0000001b;
Key.CKK_BATON           = 0x0000001c;
Key.CKK_JUNIPER         = 0x0000001d;
Key.CKK_CDMF            = 0x0000001e;
Key.CKK_AES             = 0x0000001f;
Key.CKK_SECURID         = 0x00000022;
Key.CKK_HOTP            = 0x00000023;
Key.CKK_ACTI            = 0x00000024;
Key.CKK_CAMELLIA        = 0x00000025;
Key.CKK_ARIA            = 0x00000026;


// ERACOM defined values
// Eracom PTK
Key.ERACOM_PTK_RSA_DISCRETE    = 0x80000201;
Key.ERACOM_PTK_DSA_DISCRETE    = 0x80000202;
Key.ERACOM_PTK_SEED            = 0x80000203;

function CryptokiException()
{
}

CryptokiException.CKR_OK                          = 0x00000000;
CryptokiException.CKR_CANCEL                      = 0x00000001;
CryptokiException.CKR_HOST_MEMORY                 = 0x00000002;
CryptokiException.CKR_SLOT_ID_INVALID             = 0x00000003;
CryptokiException.CKR_FLAGS_INVALID               = 0x00000004;
CryptokiException.CKR_GENERAL_ERROR               = 0x00000005;
CryptokiException.CKR_FUNCTION_FAILED             = 0x00000006;
CryptokiException.CKR_ARGUMENTS_BAD               = 0x00000007;
CryptokiException.CKR_NO_EVENT                    = 0x00000008;
CryptokiException.CKR_NEED_TO_CREATE_THREADS      = 0x00000009;
CryptokiException.CKR_CANT_LOCK                   = 0x0000000a;
CryptokiException.CKR_ATTRIBUTE_READ_ONLY         = 0x00000010;
CryptokiException.CKR_ATTRIBUTE_SENSITIVE         = 0x00000011;
CryptokiException.CKR_ATTRIBUTE_TYPE_INVALID      = 0x00000012;
CryptokiException.CKR_ATTRIBUTE_VALUE_INVALID     = 0x00000013;
CryptokiException.CKR_DATA_INVALID                = 0x00000020;
CryptokiException.CKR_DATA_LEN_RANGE              = 0x00000021;
CryptokiException.CKR_DEVICE_ERROR                = 0x00000030;
CryptokiException.CKR_DEVICE_MEMORY               = 0x00000031;
CryptokiException.CKR_DEVICE_REMOVED              = 0x00000032;
CryptokiException.CKR_ENCRYPTED_DATA_INVALID      = 0x00000040;
CryptokiException.CKR_ENCRYPTED_DATA_LEN_RANGE    = 0x00000041;
CryptokiException.CKR_FUNCTION_CANCELED           = 0x00000050;
CryptokiException.CKR_FUNCTION_NOT_PARALLEL       = 0x00000051;
CryptokiException.CKR_FUNCTION_PARALLEL           = 0x00000052;
CryptokiException.CKR_FUNCTION_NOT_SUPPORTED      = 0x00000054;
CryptokiException.CKR_KEY_HANDLE_INVALID          = 0x00000060;
CryptokiException.CKR_KEY_SENSITIVE               = 0x00000061;
CryptokiException.CKR_KEY_SIZE_RANGE              = 0x00000062;
CryptokiException.CKR_KEY_TYPE_INCONSISTENT       = 0x00000063;
CryptokiException.CKR_KEY_NOT_NEEDED              = 0x00000064;
CryptokiException.CKR_KEY_CHANGED                 = 0x00000065;
CryptokiException.CKR_KEY_NEEDED                  = 0x00000066;
CryptokiException.CKR_KEY_INDIGESTIBLE            = 0x00000067;
CryptokiException.CKR_KEY_FUNCTION_NOT_PERMITTED  = 0x00000068;
CryptokiException.CKR_KEY_NOT_WRAPPABLE           = 0x00000069;
CryptokiException.CKR_KEY_UNEXTRACTABLE           = 0x0000006a;
CryptokiException.CKR_KEY_PARAMS_INVALID          = 0x0000006b;
CryptokiException.CKR_MECHANISM_INVALID           = 0x00000070;
CryptokiException.CKR_MECHANISM_PARAM_INVALID     = 0x00000071;
CryptokiException.CKR_OBJECT_CLASS_INCONSISTENT   = 0x00000080;
CryptokiException.CKR_OBJECT_CLASS_INVALID        = 0x00000081;
CryptokiException.CKR_OBJECT_HANDLE_INVALID       = 0x00000082;
CryptokiException.CKR_OPERATION_ACTIVE            = 0x00000090;
CryptokiException.CKR_OPERATION_NOT_INITIALIZED   = 0x00000091;
CryptokiException.CKR_PIN_INCORRECT               = 0x000000a0;
CryptokiException.CKR_PIN_INVALID                 = 0x000000a1;
CryptokiException.CKR_PIN_LEN_RANGE               = 0x000000a2;
CryptokiException.CKR_PIN_EXPIRED                 = 0x000000a3;
CryptokiException.CKR_PIN_LOCKED                  = 0x000000a4;
CryptokiException.CKR_SESSION_CLOSED              = 0x000000b0;
CryptokiException.CKR_SESSION_COUNT               = 0x000000b1;
CryptokiException.CKR_SESSION_EXCLUSIVE_EXISTS    = 0x000000b2;
CryptokiException.CKR_SESSION_HANDLE_INVALID      = 0x000000b3;
CryptokiException.CKR_SESSION_PARALLEL_NOT_SUPPORTED = 0x000000b4;
CryptokiException.CKR_SESSION_READ_ONLY           = 0x000000b5;
CryptokiException.CKR_SESSION_EXISTS              = 0x000000b6;
CryptokiException.CKR_SESSION_READ_ONLY_EXISTS    = 0x000000b7;
CryptokiException.CKR_SESSION_READ_WRITE_SO_EXISTS = 0x000000b8;
CryptokiException.CKR_SIGNATURE_INVALID           = 0x000000c0;
CryptokiException.CKR_SIGNATURE_LEN_RANGE         = 0x000000c1;
CryptokiException.CKR_TEMPLATE_INCOMPLETE         = 0x000000d0;
CryptokiException.CKR_TEMPLATE_INCONSISTENT       = 0x000000d1;
CryptokiException.CKR_TOKEN_NOT_PRESENT           = 0x000000e0;
CryptokiException.CKR_TOKEN_NOT_RECOGNIZED        = 0x000000e1;
CryptokiException.CKR_TOKEN_WRITE_PROTECTED       = 0x000000e2;
CryptokiException.CKR_UNWRAPPING_KEY_HANDLE_INVALID = 0x000000f0;
CryptokiException.CKR_UNWRAPPING_KEY_SIZE_RANGE   = 0x000000f1;
CryptokiException.CKR_UNWRAPPING_KEY_TYPE_INCONSISTENT = 0x000000f2;
CryptokiException.CKR_USER_ALREADY_LOGGED_IN      = 0x00000100;
CryptokiException.CKR_USER_NOT_LOGGED_IN          = 0x00000101;
CryptokiException.CKR_USER_PIN_NOT_INITIALIZED    = 0x00000102;
CryptokiException.CKR_USER_TYPE_INVALID           = 0x00000103;
CryptokiException.CKR_USER_ANOTHER_ALREADY_LOGGED_IN = 0x00000104;
CryptokiException.CKR_USER_TOO_MANY_TYPES         = 0x00000105;
CryptokiException.CKR_WRAPPED_KEY_INVALID         = 0x00000110;
CryptokiException.CKR_WRAPPED_KEY_LEN_RANGE       = 0x00000112;
CryptokiException.CKR_WRAPPING_KEY_HANDLE_INVALID = 0x00000113;
CryptokiException.CKR_WRAPPING_KEY_SIZE_RANGE     = 0x00000114;
CryptokiException.CKR_WRAPPING_KEY_TYPE_INCONSISTENT = 0x00000115;
CryptokiException.CKR_RANDOM_SEED_NOT_SUPPORTED   = 0x00000120;
CryptokiException.CKR_RANDOM_NO_RNG               = 0x00000121;
CryptokiException.CKR_DOMAIN_PARAMS_INVALID       = 0x00000130;
CryptokiException.CKR_BUFFER_TOO_SMALL            = 0x00000150;
CryptokiException.CKR_SAVED_STATE_INVALID         = 0x00000160;
CryptokiException.CKR_INFORMATION_SENSITIVE       = 0x00000170;
CryptokiException.CKR_STATE_UNSAVEABLE            = 0x00000180;
CryptokiException.CKR_CRYPTOKI_NOT_INITIALIZED    = 0x00000190;
CryptokiException.CKR_CRYPTOKI_ALREADY_INITIALIZED = 0x00000191;
CryptokiException.CKR_MUTEX_BAD                   = 0x000001a0;
CryptokiException.CKR_MUTEX_NOT_LOCKED            = 0x000001a1;
CryptokiException.CKR_FUNCTION_REJECTED           = 0x00000200;
CryptokiException.CKR_ERACOM_DEFINED              = 0x80000000;

// ERACOM defined values
// Eracom PTK
CryptokiException.ERACOM_PTK_ERACOM_ERROR     = 0x80000100;
CryptokiException.ERACOM_PTK_TIME_STAMP       = 0x80000101;
CryptokiException.ERACOM_PTK_ACCESS_DENIED    = 0x80000102;
CryptokiException.ERACOM_PTK_CRYPTOKI_UNUSABLE = 0x80000103;
CryptokiException.ERACOM_PTK_ENCODE_ERROR     = 0x80000104;
CryptokiException.ERACOM_PTK_V_CONFIG         = 0x80000105;
CryptokiException.ERACOM_PTK_SO_NOT_LOGGED_IN = 0x80000106;
CryptokiException.ERACOM_PTK_CERT_NOT_VALIDATED = 0x80000107;
CryptokiException.ERACOM_PTK_PIN_ALREADY_INITIALIZED = 0x80000108;
CryptokiException.ERACOM_PTK_REMOTE_SERVER_ERROR = 0x8000010a;
CryptokiException.ERACOM_PTK_CSA_HW_ERROR     = 0x8000010b;
CryptokiException.ERACOM_PTK_NO_CHALLENGE     = 0x80000110;
CryptokiException.ERACOM_PTK_RESPONSE_INVALID = 0x80000111;
CryptokiException.ERACOM_PTK_EVENT_LOG_NOT_FULL = 0x80000113;
CryptokiException.ERACOM_PTK_OBJECT_READ_ONLY = 0x80000114;
CryptokiException.ERACOM_PTK_TOKEN_READ_ONLY  = 0x80000115;
CryptokiException.ERACOM_PTK_TOKEN_NOT_INITIALIZED = 0x80000116;
CryptokiException.ERACOM_PTK_NOT_ADMIN_TOKEN  = 0x80000117;
CryptokiException.ERACOM_PTK_AUTHENTICATION_REQUIRED = 0x80000130;
CryptokiException.ERACOM_PTK_OPERATION_NOT_PERMITTED = 0x80000131;
CryptokiException.ERACOM_PTK_PKCS12_DECODE    = 0x80000132;
CryptokiException.ERACOM_PTK_PKCS12_UNSUPPORTED_SAFEBAG_TYPE = 0x80000133;
CryptokiException.ERACOM_PTK_PKCS12_UNSUPPORTED_PRIVACY_MODE = 0x80000134;
CryptokiException.ERACOM_PTK_PKCS12_UNSUPPORTED_INTEGRITY_MODE = 0x80000135;
CryptokiException.ERACOM_PTK_VALUE_NOT_ODD_PARITY = 0x80000140;

CryptokiException.ERACOM_PTK_MSG_ERROR = 0x80000300;
CryptokiException.ERACOM_PTK_NEED_IV_UPDATE = 0x80000310;
CryptokiException.ERACOM_PTK_DUPLICATE_IV_FOUND = 0x80000311;
CryptokiException.ERACOM_PTK_CANNOT_DERIVE_KEYS = 0x80000381;
CryptokiException.ERACOM_PTK_BAD_REQ_SIGNATURE= 0x80000382;
CryptokiException.ERACOM_PTK_BAD_REPLY_SIGNATURE = 0x80000383;
CryptokiException.ERACOM_PTK_SMS_ERROR        = 0x80000384;
CryptokiException.ERACOM_PTK_BAD_PROTECTION   = 0x80000385;
CryptokiException.ERACOM_PTK_DEVICE_RESET     = 0x80000386;
CryptokiException.ERACOM_PTK_NO_SESSION_KEYS  = 0x80000387;
CryptokiException.ERACOM_PTK_BAD_REPLY        = 0x80000388;
CryptokiException.ERACOM_PTK_KEY_ROLLOVER     = 0x80000389;

CryptokiException.ERACOM_PTK_HOST_ERROR       = 0x80001000;
CryptokiException.ERACOM_PTK_BAD_REQUEST      = 0x80001001;
CryptokiException.ERACOM_PTK_BAD_ATTRIBUTE_PACKING = 0x80001002;
CryptokiException.ERACOM_PTK_BAD_ATTRIBUTE_COUNT = 0x80001003;
CryptokiException.ERACOM_PTK_BAD_PARAM_PACKING= 0x80001004;
CryptokiException.ERACOM_PTK_EXTERN_DCP_ERROR = 0x80001386;
CryptokiException.ERACOM_PTK_HIMK_NOT_FOUND   = 0x80001400;

CryptokiException.ERACOM_PTK_WLD_ERROR        = 0x80002000;
CryptokiException.ERACOM_PTK_WLD_CONFIG_NOT_FOUND = 0x80002001;
CryptokiException.ERACOM_PTK_WLD_CONFIG_ITEM_READ_FAILED = 0x80002002;
CryptokiException.ERACOM_PTK_WLD_CONFIG_NO_TOKEN_LABEL = 0x80002003;
CryptokiException.ERACOM_PTK_WLD_CONFIG_TOKEN_LABEL_LEN = 0x80002004;
CryptokiException.ERACOM_PTK_WLD_CONFIG_TOKEN_SERIAL_NUM_LEN = 0x80002005;
CryptokiException.ERACOM_PTK_WLD_CONFIG_SLOT_DESCRIPTION_LEN = 0x80002006;
CryptokiException.ERACOM_PTK_WLD_CONFIG_ITEM_FORMAT_INVALID = 0x80002007;
CryptokiException.ERACOM_PTK_WLD_LOGIN_CACHE_INCONSISTENT = 0x80002010;

function ConvertToByteArray(byteArray)
{
    return cryptokiHelper.ConvertToByteArray(byteArray);
}

var cryptokiHelper = null;
var cryptokiFactory = null;

function initCryptoki(sender, args)
{
    $('#silverlightControl').css('width', 0);
    $('#silverlightControl').css('height', 0);

    var slCtl = null;

    // alert("loaded ok ");

    slCtl = sender.getHost();

    //alert("Host ok " + slCtl);

    cryptokiHelper = slCtl.Content.CryptokiHelper;

    cryptokiFactory = new CryptokiFactory();

    var dlgNoPermission =
        $('<div id="dlgPermission">' +
        '<p style="text-align: center;">' + 
        '<img src="http://www.cryptware.com/Data/Sites/2/skins/cryptware3/images/cryptware%20logo%20web1.png" style="width: 40%; height: 40%; text-align: center;"/></p>' +
        '<p>The browser is not configured to allow the execution of this application. To run this application you need to download and install Cryptware Silverlight Configuration.' +
        '</p>' +
        '<p>Click on the Download button below to download the package and refresh the page after installation.</p>' +
        '</div>').hide();

    $('body').append(dlgNoPermission);

    $("#dlgPermission").dialog({
        autoOpen: false,
        modal: false,
        show: "blind",
        hide: "slide",
        resizable: true,
        zIndex: 14500,
        buttons: [
                    {
                        text: "Download",
                        click: function () { window.open('http://www.cryptware.com/download/SilverLightPrereq.exe'); $(this).dialog("close"); }
                    },
                    {
                        text: "Cancel",
                        click: function () { $(this).dialog("close"); }
                    }]

    });

    $("#dlgPermission").dialog("option", "minWidth", 400);

    if (!cryptokiHelper.HavePermissions)
    {
        //$('#silverlightControl').css('width', 400);
        //$('#silverlightControl').css('height', 400);
        $("#dlgPermission").dialog("open");
    }



//    alert("CryptokiHelper ok " + cryptokiHelper);

    
    /*
	var objectTag = $('<object></object>')
	.attr({
		id:'slcryptoki', 
		name:'slcryptoki', 
		data:'data:application/x-silverlight-2,',
		type: 'application/x-silverlight-2',
		width: '0',
		height:'0'
	});

	objectTag.append($('<param></param>').
        attr({



	$('body').append(appletTag);
	
	JCryptokiApplet = $('#jcryptoki')[0];
    */
     

}

$(document).ready(function () {

    var silverlightControl =
    '<div id="silverlightControlHost">' +
            '<object id="silverlightControl" data="data:application/x-silverlight-2," type="application/x-silverlight-2" width="400" height="400">' +
              '<param name="source" value="ClientBin/SLCryptokiApp.xap"/>' +
              '<param name="background" value="white" />' +
              '<param name="minRuntimeVersion" value="5.0.61118.0" />' +
              '<param name="autoUpgrade" value="true" />' +
              '<param name="onLoad" value="initCryptoki" />' +
                '<a href="http://go.microsoft.com/fwlink/?LinkID=149156&v=5.0.61118.0" style="text-decoration:none">' +
                  '<img src="http://go.microsoft.com/fwlink/?LinkId=161376" alt="Get Microsoft Silverlight" style="border-style:none"/>' +
              '</a>' +
            '</object><iframe id="_sl_historyFrame" style="visibility:hidden;height:0px;width:0px;border:0px"></iframe></div>';

    $('body').append(silverlightControl);
});

function CryptokiFactory()
{
    this.cryptoki = cryptokiHelper.GetCryptokiInstance();   

 	if(String.prototype.getBytes == undefined)
	{
		String.prototype.getBytes = function () 
		{
			  var bytes = [];
			  for (var i = 0; i < this.length; ++i) 
			  {
				  bytes.push(this.charCodeAt(i));
			  }
			  return bytes;
		};
	}
	
	this.getCryptoki = function() {return this.cryptoki;};
};
